﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows;
using System.Xml.Serialization;
using CSHTML5;
using ButtonTest.ViewModels;
using Windows.System;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;

namespace ButtonTest
{
    public partial class MainPage : Page
    {
        private MainPageViewModel currentVM;

        public MainPageViewModel CurrentVM
        {
            get
            {
                if (currentVM == null)
                    currentVM = new MainPageViewModel();
                return currentVM;
            }
            set
            {
                if (currentVM != value)
                {
                    currentVM = value;
                    SetDataContext(currentVM);
                }
            }
        }


        /// <summary>
        /// This is the main form for the GUI. Initializes a new instance of the <see cref="MainPage" /> class.
        /// </summary>
        public MainPage()
        {
            InitializeComponent();
            
            Loaded += MainPage_Loaded;
            UserNameTextBox.Focus();
            UserNameTextBox.KeyDown += TextBox_KeyDown;
            PasswordTextBox.KeyDown += PasswordBox_KeyDown;
        }

        private async void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                // Make sure the viewmodel is created
                var throwaway = CurrentVM.UserName;

                // Make sure that the Div has an ID
                object div = Interop.GetDiv(LayoutRoot);
                LoginButton.Click += BtnLogInClick;
            }
            catch (Exception ex)
            {
                DisplayErrorMessageWithInner(ex.Message);
            }
        }

        /// <summary>
        /// Displays and logs the error message with the inner exception.
        /// </summary>
        /// <param name="msg">The MSG.</param>
        private void DisplayErrorMessageWithInner(string msg)
        {
            MessageBox.Show(msg);
        }

        /// <summary>
        /// Handles the login button click
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs" /> instance containing the event data.</param>
        public void BtnLogInClick(object sender, RoutedEventArgs e)
        {
            Debug.WriteLine($"{DateTime.Now} - Login button clicked");
        }

        /// <summary>
        /// Handles the KeyDown event of the TextBox control. Commits text box values when the user presses ENTER.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Input.KeyEventArgs" /> instance containing the event data.</param>
        private void TextBox_KeyDown(object sender, KeyRoutedEventArgs e)
        {
            var tb = sender as TextBox;
            if (e.Key == VirtualKey.Enter || e.Key == VirtualKey.Tab)
            {
                tb?.GetBindingExpression(TextBox.TextProperty).UpdateSource();
                //if (e.Key == System.Windows.Input.Key.Enter)
                //    BtnLogInClick(sender, e); <-- Don't do this here. It is already being done by the DefaultButtonService attached to the textbox.
            }
        }

        /// <summary>
        /// Handles the KeyDown event of the PasswordBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Input.KeyEventArgs" /> instance containing the event data.</param>
        private void PasswordBox_KeyDown(object sender, KeyRoutedEventArgs e)
        {
            var tb = sender as PasswordBox;
            if (e.Key == VirtualKey.Enter || e.Key == VirtualKey.Tab)
            {
                tb?.GetBindingExpression(PasswordBox.PasswordProperty).UpdateSource();
                //if (e.Key == System.Windows.Input.Key.Enter)
                //    BtnLogInClick(sender, e); <-- Don't do this here. It is already being done by the DefaultButtonService attached to the password box.
            }
        }

        public void SetDataContext(MainPageViewModel vm)
        {
            LayoutRoot.DataContext = vm;
        }
    }
}